use sea_orm_migration::prelude::*;

#[derive(DeriveMigrationName)]
pub struct Migration;

#[async_trait::async_trait]
impl MigrationTrait for Migration {
    async fn up(&self, manager: &SchemaManager) -> Result<(), DbErr> {
        manager
            .create_table(
                Table::create()
                    .table(User::Table)
                    .if_not_exists()
                    .col(ColumnDef::new(User::Id)
                        .uuid()
                        .unique_key()
                        .not_null()
                        .primary_key()
                    )
                    .col(ColumnDef::new(User::Username)
                        .string()
                        .unique_key()
                        .not_null()
                    )
                    .col(ColumnDef::new(User::Email)
                        .string()
                        .unique_key()
                        .not_null()
                    )
                    .col(ColumnDef::new(User::PwdHash)
                        .string()
                        .not_null()
                    )
                    .col(ColumnDef::new(User::PwdSalt)
                        .string()
                        .not_null()
                    )
                    .col(ColumnDef::new(User::IsAdministrator)
                        .boolean()
                        .not_null()
                    )
                    .col(ColumnDef::new(User::CreatedAt)
                        .date_time()
                        .not_null()
                    )
                    .to_owned(),
            )
            .await
    }

    async fn down(&self, manager: &SchemaManager) -> Result<(), DbErr> {
        manager
            .drop_table(Table::drop().table(User::Table).to_owned())
            .await
    }
}

#[derive(DeriveIden)]
enum User {
    Table,
    Id,
    Username,
    Email,
    PwdHash,
    PwdSalt,
    IsAdministrator,
    CreatedAt
}
