use sea_orm_migration::prelude::*;

#[derive(DeriveMigrationName)]
pub struct Migration;

#[async_trait::async_trait]
impl MigrationTrait for Migration {
    async fn up(&self, manager: &SchemaManager) -> Result<(), DbErr> {
        // Replace the sample below with your own migration scripts
        manager
            .create_table(
                Table::create()
                    .table(Computer::Table)
                    .if_not_exists()
                    .col(ColumnDef::new(Computer::Id)
                        .uuid()
                        .not_null()
                        .primary_key()
                    )
                    .col(ColumnDef::new(Computer::Name)
                        .string()
                        .unique_key()
                        .not_null()
                    )
                    .col(ColumnDef::new(Computer::Group)
                        .string()
                        .not_null()
                    )
                    .col(ColumnDef::new(Computer::Type)
                        .string()
                        .not_null()
                    )
                    .col(ColumnDef::new(Computer::AccessToken)
                        .string()
                        .unique_key()
                        .not_null()
                    )
                    .col(ColumnDef::new(Computer::CreatedAt)
                        .date_time()
                        .not_null()
                    )
                    .to_owned(),
            )
            .await
    }

    async fn down(&self, manager: &SchemaManager) -> Result<(), DbErr> {
        manager
            .drop_table(Table::drop().table(Computer::Table).to_owned())
            .await
    }
}

#[derive(DeriveIden)]
enum Computer {
    Table,
    Id,
    Name,
    Group,
    Type,
    AccessToken,
    CreatedAt
}
