use sea_orm_migration::{prelude::*, schema::*};

#[derive(DeriveMigrationName)]
pub struct Migration;

#[async_trait::async_trait]
impl MigrationTrait for Migration {
    async fn up(&self, manager: &SchemaManager) -> Result<(), DbErr> {
        manager
            .create_table(
                Table::create()
                    .table(PcGroup::Table)
                    .if_not_exists()
                    .col(ColumnDef::new(PcGroup::Id)
                        .uuid()
                        .unique_key()
                        .primary_key()
                        .not_null()
                    )
                    .col(ColumnDef::new(PcGroup::Name)
                        .string()
                        .not_null()
                    )
                    .col(ColumnDef::new(PcGroup::AvailableActions)
                        .json()
                        .not_null()
                    )
                    .to_owned(),
            )
            .await
    }

    async fn down(&self, manager: &SchemaManager) -> Result<(), DbErr> {
        manager
            .drop_table(Table::drop().table(PcGroup::Table).to_owned())
            .await
    }
}

#[derive(DeriveIden)]
enum PcGroup {
    Table,
    Id,
    Name,
    AvailableActions,
}
