use std::{collections::HashMap, sync::mpsc::{self, Receiver, Sender}};

use axum::{body::HttpBody, extract::{ws::WebSocket, State, WebSocketUpgrade}, http::StatusCode, response::Response};
use axum_extra::TypedHeader;
use chrono::{DateTime, Utc};
use futures_util::{stream::{SplitSink, SplitStream}, StreamExt};
use headers::{authorization::Bearer, Authorization};
use sea_orm::prelude::Uuid;
use serde::{Deserialize, Serialize};
use tokio::sync::Mutex;

use crate::context::AppContext;

lazy_static::lazy_static!(
    pub static ref WS_CLIENTS: Mutex<HashMap<Uuid, WsClient>> = Mutex::new(HashMap::new());
);

#[derive(Debug)]
pub struct WsClient {
    rx: Receiver<WsMessage>,
    tx: Sender<WsMessage>,
    alive: bool,
    last_heartbeat: DateTime<Utc>
}

impl WsClient {
    pub fn new(rx: Receiver<WsMessage>, tx: Sender<WsMessage>) -> Self {
        Self {
            rx, tx, 
            alive: false,
            last_heartbeat: chrono::Utc::now()
        }
    }
}

#[derive(Debug, Clone)]
pub struct WsHandler {
}

impl WsHandler {
    pub fn new() -> Self {
        Self {

        }
    }
    pub async fn ws_handler(ws: WebSocketUpgrade, State(ctx): State<AppContext>, TypedHeader(token): TypedHeader<Authorization<Bearer>>) -> Response {
        let token = token.0.token().to_string();
        let Ok(token) = Uuid::parse_str(&token) else {
            todo!()
            // return Response::status(Body)
        };
        ws.on_upgrade(move |socket| Self::handle_socket(socket, token))
    }

    async fn handle_socket(socket: WebSocket, token: Uuid) {
        let (sender, receiver) = socket.split();
        let (r_tx, from_socket) = mpsc::channel();
        let (to_socket, s_rx) = mpsc::channel();

        async fn send(sender: SplitSink<WebSocket, axum::extract::ws::Message>, s_rx: mpsc::Receiver<WsMessage>) {
            
        }

        async fn recv(mut receiver: SplitStream<WebSocket>, tx: mpsc::Sender<WsMessage>) {
            while let Some(msg) = receiver.next().await {
                let Ok(msg) = msg else {
                    return;
                };
                let Ok(msg) = msg.to_text() else {
                    return;
                };

            }
        }

        tokio::spawn(recv(receiver, r_tx));
        tokio::spawn(send(sender, s_rx));
        Self::add_client(token, from_socket, to_socket).await;

    }

    async fn add_client(token: Uuid, rx: Receiver<WsMessage>, tx: Sender<WsMessage>) {
        WS_CLIENTS.lock().await.insert(token, WsClient::new(rx, tx));
    }
}



#[derive(Debug, Clone, Serialize, Deserialize)]
pub struct WsMessage {
    pub ws_msg_t: usize,
    pub ws_msg: Message,
}

#[derive(Debug, Clone, Serialize, Deserialize)]
pub struct Message {
    pub auth: Uuid,
    pub msg: MessageType
}

#[derive(Debug, Clone, Serialize, Deserialize)]
pub enum MessageType {
    Keypad(KeypadMessageType)
}

#[derive(Debug, Clone, Serialize, Deserialize)]
pub enum KeypadMessageType {
    DoorOpenEv {
        user: Uuid,
        timestamp: usize,
        door_id: Uuid
    }
}



