use std::time::Duration;
use sea_orm::{ConnectOptions, Database as SODatabase, DatabaseConnection};

mod models;

// to update the models run `sea-orm-cli generate entity -o src/db/models`

#[derive(Debug, Clone)]
pub struct Database {
    db: DatabaseConnection
}

impl Database {
    pub fn new() -> Self {
        Self {
            db: DatabaseConnection::Disconnected
        }
    
    }
    pub async fn connect(&mut self) -> anyhow::Result<()> {
        log::info!("Connecting to SQlite database at ./db.sqlite");
        let mut opt = ConnectOptions::new("sqlite://db.sqlite?mode=rwc");
        opt.max_connections(100)
            .min_connections(5)
            .connect_timeout(Duration::from_secs(8))
            .acquire_timeout(Duration::from_secs(8))
            .idle_timeout(Duration::from_secs(8))
            .max_lifetime(Duration::from_secs(8))
            .sqlx_logging(true)
            .sqlx_logging_level(log::LevelFilter::Debug);
        self.db = SODatabase::connect(opt).await?;
        log::info!("Connection successful");
        Ok(())
    }

}
